'use strict'

var Promise = require('bluebird')
var fs = Promise.promisifyAll(require('fs'))
var path = require('path')
var os = require('os')
var _ = require('lodash')

module.exports.configFilePath = function () {
  return path.join(os.homedir(), '.share')
}
var config = module.exports

var cachedConfiguration

module.exports.get = function() {
  if (cachedConfiguration)
    return Promise.resolve(cachedConfiguration)

  return config.getConfigFileAsync()
  .catch(function (err) {
    if (err.errno === -2) {
      // console.log('Config file missing, creating')
      return config.newConfigAsync()
        .then(function () { 
          cachedConfiguration = {}
          return {}
        })
    } else {
      throw err
    }
  })
}

module.exports.newConfigAsync = function() {
  return config.saveConfigFileAsync({})
}

module.exports.getConfigFileAsync = function() {
  // console.log('reading config from file')
  return fs.readFileAsync(module.exports.configFilePath(),'utf8')
    .then(JSON.parse)
    .tap( configuration => cachedConfiguration = configuration)
}

module.exports.saveConfigFileAsync = function(configuration) {
  return fs.writeFileAsync(module.exports.configFilePath(), JSON.stringify(configuration))
    .tap( () => cachedConfiguration = configuration)
}

module.exports.merge = function(incomingConfig) {
  config.get().then(function(currentConfig) {
    var newConfig = {}
    _.merge(newConfig, currentConfig)
    _.merge(newConfig, incomingConfig)
    return config.saveConfigFileAsync(newConfig)
  })
}