'use strict'

var Promise = require('bluebird')
var readline = require('readline')
Promise.promisifyAll(readline.Interface.prototype)
var config = require('./config')

var attributes = [
  { name: 'awsAccountName', type: 'string' },
  { name: 's3Bucket', type: 'string' },
  { name: 'slackWebHookUrl', type: 'string' },
  { name: 'userName', type: 'string' },
  { name: 'channels', type: 'commalist' },
  { name: 'userNames', type: 'commalist' }
]

function showHelp () {
  console.log(`
Usage: share configure help           show this message
       share configure cfgstring      apply a base configuration
       share configure                update configure

`)
}

var convert = {
  string: function (string) { return string },
  number: parseInt,
  commalist: function (string) { return string.split(',') }
}

function showConfiguration(configuration) {
  console.log(`Configuration:
---------------------`)
  var maxLength = Math.max.apply(Math, attributes.map( a => a.name.length ))
  attributes.forEach( function(a) {
    console.log(a.name + '                                           '.slice(0, maxLength - a.name.length), ':', configuration[a.name])
  })
  console.log()
}

function question(ri, prompt) {
  return new Promise(function(resolve) { 
    ri.question(prompt,resolve)
  })
}

function configure(currentConfiguration) {
  var newConfig = {}
  var ri = readline.createInterface({
    input: process.stdin,
    output: process.stdout
  })

  Promise.mapSeries(attributes, attribute => {
    return question(ri, `${attribute.name} [${currentConfiguration[attribute.name]}]:`)
    .then(result => {
      if (result === '')
        newConfig[attribute.name] = currentConfiguration[attribute.name]
      else
        newConfig[attribute.name] = convert[attribute.type](result)
    })
  })
  .then( () => ri.close())
  .then( () => config.saveConfigFileAsync(newConfig))
}


module.exports = function (options) {

  switch (options.arguments[0]) {
    case 'show':
      return config.get().then(showConfiguration)
      break
    case 'help':
      return showHelp()
      break
    case undefined:
      return config.get().then(configure)
      break
    default:
      var baseConfig = JSON.parse(new Buffer(options.arguments[0], 'base64').toString('ascii'))
      config.merge(baseConfig)
      break
  }
}

// share configure 
//   show current and prompt for new

// share configure show
//   show the currenct config

// share configure help
//   show the help

//save configuration as a .json
//display config
//
// 

// options:
//   s3bucket
//   encryption?
//   slack config stuff