'use strict'

var Promise = require('bluebird')
var uuid = require('uuid')
var aws = require('aws-sdk')
var s3  = Promise.promisifyAll(new aws.S3())
var config = require('./config.js')

var makeRandomKey = function () {
  return uuid.v4().slice(0, 8)
}

function validateConfig(configuration) {
  if (!configuration.s3Bucket)
    throw { error: 'bucket not configured' }
  return configuration
}

// module.exports.createBucket = function (bucket) {

// }

module.exports.uploadSecret = function(data, datatype) {
  var key = makeRandomKey()
  return config.get()
  .then(validateConfig)
  .then(function(configuration) {
    return s3.putObjectAsync({
      Bucket: configuration.s3Bucket,
      Key: key,
      ServerSideEncryption: undefined, // set for encryption
      Body: data,
      Metadata: { datatype: datatype }
    })
  })
  .then(function () { return { key: key } } )
}

module.exports.downloadSecret = function(key) {
  return config.get()
  .then(validateConfig)
  .then(function(configuration) {
    return s3.getObjectAsync({
      Bucket: configuration.s3Bucket,
      Key: key,
    })
  })
  .then(function(result) {
    return {
      data: result.Body,
      datatype: result.Metadata.datatype
    }
  })
}