'use strict'

var Promise = require('bluebird')
var config = require('./config.js')
var request = require('request')

// need webhook url
// username

function notify(target, message, config) {
  return new Promise.fromCallback( cb => request.post({
      url: config.slackWebHookUrl,
      body: JSON.stringify({
        text: message,
        username: 'ShareBot on behalf of ' + config.userName,
        'icon_emoji': ':shamrock:',
        'channel': target
      })
    }, cb)
  )
}

function checkTargets(targets, config, options) {
  return targets.map( t => config.channels.indexOf(t) >= 0 ? '#' + t : t)
                .map( t => config.userNames.indexOf(t) >= 0 ? '@' + t : t)
                .filter( t => {
                  var valid = t.startsWith('#') || t.startsWith('@')
                  if (!valid) console.log('Skipping invalid channel target:', t)
                  return valid
                })
                .filter( t => t.startsWith('@') || !options.usersOnly)
}

module.exports.notifySecret = function(message, targets, options) {
  options = options || {}
  return config.get()
  .then(function (config) {
    var validTargets = checkTargets(targets, config, options)
    return Promise.map(validTargets, function (target) {
      return notify(target, message, config)
    })
  })
}
